@@ -132,8 +132,9 @@ pad_process_absolute(struct pad_dispatch *pad,
 		pad->have_abs_misc_terminator = true;
 		break;
 	default:
-		log_info(pad_libinput_context(pad),
-			 "Unhandled EV_ABS event code %#x\n", e->code);
+		evdev_log_info(device,
+			       "Unhandled EV_ABS event code %#x\n",
+			       e->code);
 		break;
 	}
 }
@@ -452,7 +453,7 @@ pad_process(struct evdev_dispatch *dispatch,
 	    struct input_event *e,
 	    uint64_t time)
 {
-	struct pad_dispatch *pad = (struct pad_dispatch *)dispatch;
+	struct pad_dispatch *pad = pad_dispatch(dispatch);
 
 	switch (e->type) {
 	case EV_ABS:
@@ -469,10 +470,10 @@ pad_process(struct evdev_dispatch *dispatch,
 		 * now */
 		break;
 	default:
-		log_error(pad_libinput_context(pad),
-			  "Unexpected event type %s (%#x)\n",
-			  libevdev_event_type_get_name(e->type),
-			  e->type);
+		evdev_log_error(device,
+				"Unexpected event type %s (%#x)\n",
+				libevdev_event_type_get_name(e->type),
+				e->type);
 		break;
 	}
 }
@@ -481,7 +482,7 @@ static void
 pad_suspend(struct evdev_dispatch *dispatch,
 	    struct evdev_device *device)
 {
-	struct pad_dispatch *pad = (struct pad_dispatch *)dispatch;
+	struct pad_dispatch *pad = pad_dispatch(dispatch);
 	struct libinput *libinput = pad_libinput_context(pad);
 	unsigned int code;
 
@@ -496,7 +497,7 @@ pad_suspend(struct evdev_dispatch *dispatch,
 static void
 pad_destroy(struct evdev_dispatch *dispatch)
 {
-	struct pad_dispatch *pad = (struct pad_dispatch*)dispatch;
+	struct pad_dispatch *pad = pad_dispatch(dispatch);
 
 	pad_destroy_leds(pad);
 	free(pad);
@@ -542,6 +543,11 @@ pad_init_buttons(struct pad_dispatch *pad,
 			pad->button_map[code] = map++;
 	}
 
+	for (code = BTN_LEFT; code < BTN_LEFT + 7; code++) {
+		if (libevdev_has_event_code(device->evdev, EV_KEY, code))
+			pad->button_map[code] = map++;
+	}
+
 	pad->nbuttons = map;
 }
 
@@ -556,6 +562,7 @@ pad_init_left_handed(struct evdev_device *device)
 static int
 pad_init(struct pad_dispatch *pad, struct evdev_device *device)
 {
+	pad->base.dispatch_type = DISPATCH_TABLET_PAD;
 	pad->base.interface = &pad_interface;
 	pad->device = device;
 	pad->status = PAD_NONE;
@@ -579,7 +586,7 @@ static enum libinput_config_status
 pad_sendevents_set_mode(struct libinput_device *device,
 			enum libinput_config_send_events_mode mode)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct pad_dispatch *pad = (struct pad_dispatch*)evdev->dispatch;
 
 	if (mode == pad->sendevents.current_mode)
@@ -603,7 +610,7 @@ pad_sendevents_set_mode(struct libinput_device *device,
 static enum libinput_config_send_events_mode
 pad_sendevents_get_mode(struct libinput_device *device)
 {
-	struct evdev_device *evdev = (struct evdev_device*)device;
+	struct evdev_device *evdev = evdev_device(device);
 	struct pad_dispatch *dispatch = (struct pad_dispatch*)evdev->dispatch;
 
 	return dispatch->sendevents.current_mode;
