@@ -90,9 +90,6 @@ static const struct evdev_udev_tag_match evdev_udev_tag_matches[] = {
 	{"ID_INPUT_POINTINGSTICK",	EVDEV_UDEV_TAG_POINTINGSTICK},
 	{"ID_INPUT_TRACKBALL",		EVDEV_UDEV_TAG_TRACKBALL},
 	{"ID_INPUT_SWITCH",		EVDEV_UDEV_TAG_SWITCH},
-
-	/* sentinel value */
-	{ 0 },
 };
 
 static inline bool
@@ -1092,10 +1089,25 @@ evdev_tag_trackpoint(struct evdev_device *device,
 		device->tags |= EVDEV_TAG_TRACKPOINT;
 }
 
+static inline void
+evdev_tag_keyboard_internal(struct evdev_device *device)
+{
+	device->tags |= EVDEV_TAG_INTERNAL_KEYBOARD;
+	device->tags &= ~EVDEV_TAG_EXTERNAL_KEYBOARD;
+}
+
+static inline void
+evdev_tag_keyboard_external(struct evdev_device *device)
+{
+	device->tags |= EVDEV_TAG_EXTERNAL_KEYBOARD;
+	device->tags &= ~EVDEV_TAG_INTERNAL_KEYBOARD;
+}
+
 static void
 evdev_tag_keyboard(struct evdev_device *device,
 		   struct udev_device *udev_device)
 {
+	const char *prop;
 	int code;
 
 	if (!libevdev_has_event_type(device->evdev, EV_KEY))
@@ -1108,6 +1120,21 @@ evdev_tag_keyboard(struct evdev_device *device,
 			return;
 	}
 
+	/* This should eventually become ID_INPUT_KEYBOARD_INTEGRATION */
+	prop = udev_device_get_property_value(udev_device,
+					      "LIBINPUT_ATTR_KEYBOARD_INTEGRATION");
+	if (prop) {
+		if (streq(prop, "internal")) {
+			evdev_tag_keyboard_internal(device);
+		} else if (streq(prop, "external")) {
+			evdev_tag_keyboard_external(device);
+		} else {
+			evdev_log_info(device,
+				       "tagged with unknown value %s\n",
+				       prop);
+		}
+	}
+
 	device->tags |= EVDEV_TAG_KEYBOARD;
 }
 
@@ -1536,10 +1563,19 @@ static uint32_t
 evdev_scroll_get_default_button(struct libinput_device *device)
 {
 	struct evdev_device *evdev = evdev_device(device);
+	unsigned int code;
 
 	if (libevdev_has_event_code(evdev->evdev, EV_KEY, BTN_MIDDLE))
 		return BTN_MIDDLE;
 
+	for (code = BTN_SIDE; code <= BTN_TASK; code++) {
+		if (libevdev_has_event_code(evdev->evdev, EV_KEY, code))
+			return code;
+	}
+
+	if (libevdev_has_event_code(evdev->evdev, EV_KEY, BTN_RIGHT))
+		return BTN_RIGHT;
+
 	return 0;
 }
 
@@ -1851,7 +1887,7 @@ static inline void
 evdev_process_event(struct evdev_device *device, struct input_event *e)
 {
 	struct evdev_dispatch *dispatch = device->dispatch;
-	uint64_t time = s2us(e->time.tv_sec) + e->time.tv_usec;
+	uint64_t time = tv2us(&e->time);
 
 #if 0
 	if (libevdev_event_is_code(e, EV_SYN, SYN_REPORT))
@@ -2245,9 +2281,7 @@ evdev_read_model_flags(struct evdev_device *device)
 		MODEL(ALPS_TOUCHPAD),
 		MODEL(SYNAPTICS_SERIAL_TOUCHPAD),
 		MODEL(JUMPING_SEMI_MT),
-		MODEL(APPLE_INTERNAL_KEYBOARD),
 		MODEL(CYBORG_RAT),
-		MODEL(CYAPA),
 		MODEL(HP_STREAM11_TOUCHPAD),
 		MODEL(LENOVO_T450_TOUCHPAD),
 		MODEL(TOUCHPAD_VISIBLE_MARKER),
@@ -2364,18 +2398,16 @@ evdev_device_get_udev_tags(struct evdev_device *device,
 			   struct udev_device *udev_device)
 {
 	enum evdev_device_udev_tags tags = 0;
-	const struct evdev_udev_tag_match *match;
 	int i;
 
 	for (i = 0; i < 2 && udev_device; i++) {
-		match = evdev_udev_tag_matches;
-		while (match->name) {
+		unsigned j;
+		for (j = 0; j < ARRAY_LENGTH(evdev_udev_tag_matches); j++) {
+			const struct evdev_udev_tag_match match = evdev_udev_tag_matches[j];
 			if (parse_udev_flag(device,
 					    udev_device,
-					    match->name))
-				tags |= match->tag;
-
-			match++;
+					    match.name))
+				tags |= match.tag;
 		}
 		udev_device = udev_device_get_parent(udev_device);
 	}
@@ -2835,6 +2867,38 @@ evdev_pre_configure_model_quirks(struct evdev_device *device)
 		libevdev_disable_event_code(device->evdev, EV_KEY, BTN_MIDDLE);
 }
 
+static void
+libevdev_log_func(const struct libevdev *evdev,
+		  enum libevdev_log_priority priority,
+		  void *data,
+		  const char *file,
+		  int line,
+		  const char *func,
+		  const char *format,
+		  va_list args)
+{
+	struct libinput *libinput = data;
+	enum libinput_log_priority pri = LIBEVDEV_LOG_ERROR;
+	const char prefix[] = "libevdev: ";
+	char fmt[strlen(format) + strlen(prefix) + 1];
+
+	switch (priority) {
+	case LIBEVDEV_LOG_ERROR:
+		pri = LIBINPUT_LOG_PRIORITY_ERROR;
+		break;
+	case LIBEVDEV_LOG_INFO:
+		pri = LIBINPUT_LOG_PRIORITY_INFO;
+		break;
+	case LIBEVDEV_LOG_DEBUG:
+		pri = LIBINPUT_LOG_PRIORITY_DEBUG;
+		break;
+	}
+
+	snprintf(fmt, sizeof(fmt), "%s%s", prefix, format);
+
+	log_msg_va(libinput, pri, fmt, args);
+}
+
 struct evdev_device *
 evdev_device_create(struct libinput_seat *seat,
 		    struct udev_device *udev_device)
@@ -2878,7 +2942,10 @@ evdev_device_create(struct libinput_seat *seat,
 		goto err;
 
 	libevdev_set_clock_id(device->evdev, CLOCK_MONOTONIC);
-
+	libevdev_set_device_log_function(device->evdev,
+					 libevdev_log_func,
+					 LIBEVDEV_LOG_ERROR,
+					 libinput);
 	device->seat_caps = 0;
 	device->is_mt = 0;
 	device->mtdev = NULL;
